#!/usr/local/bin/perl

##########################################################################
# mail-to-fax filter 
# Copyright (C) Harald Milz (hm@seneca.muc.de) 1996
# parses multipart/mixed files i.a.w. RFC 1341
# 
# prerequisites: 
# - mgetty+sendfax, latest version
# - metamail package, esp. mmencode and richtext
# - certainly Perl, preferably version 5
# 
# will be called through /etc/aliases
# 
# for debug purposes, there is a "-d" flag. program will 
# spit out lots of internal data and not do anything real except 
# creating the original files. 
# 
# FIXME: 
# - security check needed (via PGP signature? password? table check?)
##########################################################################

##########################################################################
# WARNING:
# as pointed out in RFC1341 pp. 41, allowing to send arbitrary
# postscript files to an interpreter like ghostscript can 
# cause severe security issues. 
##########################################################################

require "getopts.pl";

##########################################################################
# adjust several helpers here:
##########################################################################

$mailer = "/usr/lib/sendmail";
$path = $ENV{PATH} . ":/usr/local/bin:/usr/TeX/bin";
$faxspool_cmd = "PATH=$path /usr/local/bin/faxspool -q";
$mmencode = "/usr/local/bin/mmencode";
$rm_cmd = "/bin/rm -f ";
$tmpdir = "/var/spool/fax/tmp";
$logfile = "/var/spool/fax/faxmaillog";
$faxallow = "/usr/local/etc/mgetty+sendfax/fax.allow";

##########################################################################
# end of config section
##########################################################################

$header = 1;
chdir ($tmpdir);
srand ($$);
$uniquedir = rand ($$);
mkdir ("$uniquedir", 0700);			# create unique directory
chdir ("$uniquedir");

##########################################################################
# subroutine reply - sends an e-mail to the fax sender
##########################################################################

sub reply {
    local ($answer) = (@_);
    open (MAIL, "|$mailer $from") || die ("cannot exec $mailer: $!");
    printf MAIL ("From: fax (mail2fax gateway)\n");
    printf MAIL ("Subject: your fax to %s (%s)\n", $phone, $recipient);
    print MAIL;
    printf MAIL ("%s\n\n", $answer);
    print MAIL "your friendly fax server";
    close MAIL;
}

sub allowed {
    local ($fromadr) = (@_);
    open (ALLOW, "<$faxallow") || 
	die ("cannot open $faxallow for reading: $!");
    while (<ALLOW>) {
	if (/^$fromadr\s*$/) { # allow for trailing whitespace
	    printf LOG ("allowed to fax: %s\n", $from);
	    close ALLOW;
	    return (1);
	}
    }
    printf LOG ("not allowed to fax: %s\n", $from);
    close ALLOW;
    return (0);
}


($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);

&Getopts ("d");

if ($opt_d) {
    open (LOG, ">&STDERR");
} else {
    open (LOG, ">>$logfile") || die ("cannot open $logfile: $!\n");
}

printf LOG ("\n%02d.%02d.%02d %02d:%02d:%02d\n\n", $mday, 
	    $mon+1, $year, $hour, $min, $sec);
printf LOG ("faxspool_cmd = %s\n", $faxspool_cmd);

while (<>) {
    if ($header eq 1) {		# header part of message
	#      From: hm (Harald Milz)    From: Harald Milz <hm>
	if (m/^From:\s(\S+)\s\(.*/ || m/^From:[^\<]+\<(\S+)\>/) { 
	    $from = $1;
	    if (!&allowed ($from)) {
		&reply ("you are not allowed to use the mail2fax gateway.");
		exit (1);
	    }
	    $faxspool_cmd .= " -f $from ";
	    printf LOG ("from: %s\n", $from);
	    printf LOG ("faxspool_cmd = %s\n", $faxspool_cmd);
	} 
	elsif (m/^Message-ID: \b(\S+@\S+)\b/) {
	    $messageid = $1;
	}
	# Subject: "$RES" -D "$DEST" "$PHONE"
	# phone number may contain all characters except double quotes
#	elsif (m/^Subject:\s/) {
	# not used right now.
#	}
	elsif (m/^Content-Type: multipart\/mixed; boundary=\"(.*)\"/) {
	    $boundary = $1;
	} 
	elsif (/^$/) {
	    $header = 0;	# out of header
	}
    } 
    else {			# body part of message
	if (!$boundary) {
	    &reply ("your message <" . $messageid . 
		    ">  is not a MIME multipart/mixed message\n" . 
		    "it is rejected.");
	    exit (1);
	}
	if (/^--$boundary/) {	# allow for "--$boundary--" delimiters
	    # if a boundary is reached and a filename is set, this 
	    # file has just been processed. close and decode if necessary
	    if ($filename) {
		close FILE;
	        if ($content_transfer_encoding eq "base64" || 
		    $content_transfer_encoding eq "quoted-printable") {
		    system ($cmd);
		}
	    }
	}
	last if (/^--$boundary--$/); # this was the last one (epilogue)
	if (m/^Content-Type: ([^;]*);?(.*)$/) {
	    $content_type = $1;
	    printf LOG ("$content_type\n");
	    $charset = $2;
	} 
	elsif (m/^Content-Transfer-Encoding: (.*)$/) {
	    $content_transfer_encoding = $1;
	} 

	##################################################################
	# because the faxspool command line is assembled here, the 
	# sequence of the next three headers is significant
	##################################################################

	elsif (m/^Fax-Res: (.*)$/) {
	    $res = $1;
	    $faxspool_cmd .= " $res ";
	} 
	elsif (m/^Fax-Dest: (.*)$/) {
	    $recipient = $1;
	    $faxspool_cmd .= " -D \"$recipient\" ";
	} 
	elsif (m/^Fax-No: (.*)$/) {
	    ($phone = $1) =~ s/\D//g;
	    $faxspool_cmd .= " $phone ";
	} 
	# for the non-tsfax-users. This works especially fine from 
	# Netscape. See sample.mail
	elsif (m/^Fax-Text:\s*$/) {
	    $filename = "fax-header.txt";
	    open (FILE, ">$filename") || 
		die ("cannot open $filename for writing: $!");
	    $faxspool_cmd .= $filename . " " ;
            $rm_cmd .= $filename . " " ;
	}
	elsif (m/.*filename=\"([^\"]+)/) {
	    $filename = $1; 
            printf LOG ("filename = %s\n", $filename);

            ##############################################################
            # if you e-mail a HTML page or a GIF file from Netscape and 
            # convert it to Postscript before sending it, the Content-Type 
            # will be set to "application/postscript" but the filename 
            # will be unchanged. Ummmm... so we append .ps for faxspool's
	    # sake if the filename is not *.ps
            ##############################################################

            if ($content_type eq "application/postscript" && 
		$filename !~ /.*\.ps$/) {
                $filename .= ".ps";
            }
	    
            ##############################################################
            # text/richtext requires special handling, i.e. piping through
            # richtext. 
            ##############################################################

	    $faxspool_cmd .= $filename . " " ;
            $rm_cmd .= $filename . " " ;

	    ##############################################################
	    # special handling for base64 and quoted-printable
	    # all other (binary, 7bit, 8bit) should not require this
	    # x-token is not handled here. 
	    ##############################################################

            if ($content_transfer_encoding eq "base64") {
                $cmd = "$mmencode -u -p > $filename";
                $filename .= ".mm";
                $rm_cmd .= $filename . " " ;
                $cmd .= " < $filename";
		printf LOG ("cmd = %s\n", $cmd);
            } elsif ($content_transfer_encoding eq "quoted-printable") {
                $cmd = "$mmencode -u -q > $filename";
                $filename .= ".mm";
                $rm_cmd .= $filename . " " ;
                $cmd .= " < $filename";
		printf LOG ("cmd = %s\n", $cmd);
            } 
	    printf LOG ("faxspool_cmd = %s\n", $faxspool_cmd);
	    open (FILE, ">$filename") || 
		die ("cannot open $filename for writing: $!");
	}
	else {
	    print FILE;
	    # empty lines don't harm Postscript or base64 files. 
	}
    }
}

printf LOG ("Ende: faxspool_cmd = %s\n", $faxspool_cmd);
printf LOG ("Ende: rm_cmd = %s\n", $rm_cmd);
close LOG;

if (!$opt_d) {
    system ($faxspool_cmd);
    system ($rm_cmd);
    chdir ("..");
    rmdir ("$uniquedir");
    &reply ("everything OK");
}

exit (0);
